# compute 95% CIs for a mean
ci = function(data){return(t.test(data)$conf.int[1:2])}


SDTindices = function(d){
# compute discrimination SDT indices
# for this discrimination performance, awareness is irrelevant!
# consider Figure 2 in Morales et al.
# Here, high freq is considered as the signal whereas low freq is the noise
# (In terms of Macmillan & Creelman, 2005, high freq = S2 and low freq = S1)
# low  freq = stimulus A in figure 2 (left stimulus)
# high freq = stimulus B in figure 2 (right stimulus)
# With this labeling, the plotting matches that in M&C2005. Thus, criterion c is in the correct direction.

# hit rate = P('Yes'|stimulus) - proportion of yes when stimulus present
# proportion of high freq trials that are labeled (correctly) as high freq
d$Hr = (d$ACH+d$UCH)/(d$ACH+d$UCH+d$AIH+d$UIH)

# false alarm rate = P('Yes'|noise) - proportion of yes when noise
# proportion of low freq trials that are labeled (incorrectly) as high freq
d$Fr = (d$AIL+d$UIL)/(d$ACL+d$UCL+d$AIL+d$UIL)

# D prime z(H) - z(F)
d$dprime = qnorm(d$Hr) - qnorm(d$Fr)
# Criterion - 0.5 * (z(H) + z(F))
d$c = - 0.5 * (qnorm(d$Hr) + qnorm(d$Fr))
# Positive value means conservative for high freq (tendency not to respond high)
# Negative value means liberal for high freq  (tendency to respond high)
return(d)
}

MoAwThre = function(d){
  # compute discrimination awareness thresholds (z scores) for low and high freq
  # consider Figure 2 in Morales et al.

  # compute proportions
  d$propACL = d$ACL / (d$ACL+d$UCL+d$AIL+d$UIL) # AcL / L
  d$propAIH = d$AIH / (d$ACH+d$UCH+d$AIH+d$UIH) # AiH / H
  d$propACH = d$ACH / (d$ACH+d$UCH+d$AIH+d$UIH) # AcH / H
  d$propAIL = d$AIL / (d$ACL+d$UCL+d$AIL+d$UIL) # AiL / L
  
  # means of distributions
  # (center them on zero)
  d$muL = -d$dprime/2
  d$muH = d$dprime/2
  
  # low freq:
  # The low-freq awareness threshold is the average z score of two estimates
  # 1) aware-correct for low freq = z score of proportion of aware-correct for low freq
  # 2) aware-incorrect for high freq = z score of proportion of aware-incorrect for high freq
  # Each has its own distribution, and the z score is computed from the left tail of each distribution.
  d$atL = (qnorm(d$propACL, d$muL, lower.tail = T) + qnorm(d$propAIH, d$muH, lower.tail = T))/2 
  
  # high freq:
  # The high-freq awareness threshold is the average z score of two estimates
  # 1) aware-correct for high freq = z score of proportion of aware-correct for high freq
  # 2) aware-incorrect for low freq = z score of proportion of aware-incorrect for low freq
  # Each has its own distribution, and the z score is computed from the right tail of each distribution.
  d$atH = (qnorm(d$propACH, d$muH, lower.tail = F) + qnorm(d$propAIL, d$muL, lower.tail = F))/2
  return(d)
}

MoScale = function(d){
  # compute Morales scaling factors for each frequency
  # consider Figure 2 in Morales et al.
  
  # uses discrimination d', discrimination criterion c, and the awareness thresholds (as z scores, see MoAwThre)

  # compute weight of area
  d$awL = etruncnorm(a = -Inf,  b = d$atL, mean = d$muL)
  d$uwL = etruncnorm(a = d$atL, b = d$c,   mean = d$muL)
  d$awH = etruncnorm(a = d$atH, b = Inf,   mean = d$muH)
  d$uwH = etruncnorm(a = d$c,   b = d$atH, mean = d$muH)
  
  # compute scaling factor
  d$scaleL = d$awL/d$uwL
  d$scaleH = d$awH/d$uwH
  return(d)
}

SDTplot = function(fp, da, titleon = 1, pdfsave = 0){
  # plot SDT representation
  # input da needs to contain dprime, c, atL (awareness threshold low), and atH (high)
  # titleon = 1 generates a title
  # pdfsave = 1 to save the figure
  x = seq(-da$dprime/2 - 4, da$dprime/2 + 4, 0.01)
  low = dnorm(x, mean = -da$dprime/2) # noise
  high = dnorm(x, mean = da$dprime/2) # signal
  
  collow = "cyan"
  colhigh = "magenta"
  
  #par(mfrow=c(1,2)) #, pty="s")
  
  if (pdfsave==1){
    pdf(file.path(dir_fig, paste0("SDT_",fp,".pdf")), width = 11, height = 8)
  }
  
  par(mar=c(5,5,3,2)) # bottom, left, top, right
  
  # Signal and noise distributions
  plot(x, low, type='l', lwd = 3, col=collow, xaxt='n',
       xlab='', ylab='')
  title(xlab='Internal response', 
        ylab='Density', cex.lab=1.5)
  
  # add title
  if (titleon==1){
    title(paste('Subject = ', fp, 
                ", d' = ", format(round(da$dprime, 2), nsmall = 2),
                ', c = ', format(round(da$c, 2), nsmall = 2), 
                ', atL = ', format(round(da$atL, 2), nsmall = 2),
                ', atH = ', format(round(da$atH, 2), nsmall = 2), sep = ''), cex.main = 1.5)}
  
  axis(side=1, at=round(min(x)):round(max(x)), labels=round(min(x)):round(max(x)))
  lines(x, high, type = 'l', lwd = 3, col = colhigh)
  abline(v = da$c, lwd = 2, lty = 2)
  
  abline(v = da$atL, col = collow, lwd = 2, lty = 2)
  abline(v = da$atH, col = colhigh, lwd = 2, lty = 2)
  
  legend(x = "topleft",legend = c("Low frequency","High frequency"), col = c(collow, colhigh), lwd=3, bty = "n", cex = 1.2)
  legend(x = "topright",legend = c("Awareness threshold Low","Criterion", "Awareness threshold High"), col = c(collow, "black", colhigh), lwd= 2, lty=2, bty = "n", cex = 1.2)
  
  # # ROC value
  # plot(x = da$Fr, y = da$Hr, xlim = c(0,1), ylim = c(0,1), xlab = 'False alarm rate',
  #      ylab = 'Hit rate', main = paste('H: ',round(da$Hr,2),', F: ',round(da$Fr,2)))
  # abline(0,1)
  if (pdfsave==1){
    invisible(dev.off())
  }
}

Cscaleplot = function(tmp,atL, atH){
  # plot effects of response bias on scaling factor
  # atL = criterion low freq (left limit)
  # atH = criterion high freq (right limit)
  c = tmp$c
  
  x = seq(atL+.001, atH-0.001, 0.001)
  ds = data.frame(matrix(nrow = 0, ncol = 13))
  for (i in 1:length(x)) {
    tmp$c = x[i]
    ds = rbind(ds,tmp)
  }
  ds = MoScale(ds)
  par(mfrow=c(1,4), pty="m")
  ty1 = round(min(c(ds$uwL, ds$awL)),1)
  ty2 = round(max(c(ds$uwL, ds$awL)),1)
  plot(ds$c, ds$uwL, type = 'l', ylim = c(ty1,ty2), col = 'blue', lwd = 2,
       xlab = 'Discrimination c', ylab = 'Weight',
       main = "Weights for Low freq")
  lines(ds$c, ds$awL, col = 'red', lwd = 2)
  abline(h=0, lty=1)
  abline(v=c, lty=2, ltw = 1)
  legend(x = 'right', legend = c('A', 'U',"Subject's c"),
         col = c('red', 'blue', 'black'), lty = c(1,1,2), lwd = c(2,2,1), bty = "n", cex = 1.2)

  plot(x=ds$c,y=ds$scaleL, type = 'l', col = 'orange', xlab = "Discrimination c", ylab = "Scaling factor", main = "Scaling for Low freq", lwd=3)
  abline(h=0,lty=1)
  abline(v=c, lty=2, ltw = 1)

  plot(x=ds$c,y=ds$scaleL, type = 'l', col = 'orange', ylim = c(-20,20), xlab = "Discrimination c", ylab = "Scaling factor", main = "Scaling for Low freq (zoomed in)", lwd=3)
  abline(h=0,lty=1)
  abline(v=c, lty=2, ltw = 1)

  plot(x=ds$c,y=ds$scaleH, type = 'l', col = 'orange', xlab = "Discrimination c", ylab = "Scaling factor", main = "Scaling for High freq", lwd=3)
  abline(h=0,lty=1)
  abline(v=c, lty=2, ltw = 1)
}

ERPsimu = function(n = 5000, dprime = 1, c = 0, atL = -2, atH = 2){
  # simulate ERP data (as in Morales)

  # n = 5000
  # dp = 1
  # c = 0
  # atL = -2
  # atH = 2
  mu1 = -dprime/2 # 1 is low freq (left)
  mu2 = dprime/2  # 2 is high freq (right)
  sd1 = 1
  sd2 = 1

  # Generate trials
  t = data.frame(ir = c(rnorm(n = n, mean = mu1, sd = sd1),
                        rnorm(n = n, mean = mu2, sd = sd2)), 
                 stim = c(rep(1,n), rep(2,n)))
  
  t$disc = ifelse(t$ir > c, 2, 1)
  t$aware = 0
  t$aware[t$disc == 1 & t$ir < atL] = 1
  t$aware[t$disc == 2 & t$ir > atH] = 1
  
  trials = data.frame(ACL = sum(t$stim==1 & t$disc==1 & t$aware==1),
                      UCL = sum(t$stim==1 & t$disc==1 & t$aware==0),
                      AIL = sum(t$stim==1 & t$disc==2 & t$aware==1),
                      UIL = sum(t$stim==1 & t$disc==2 & t$aware==0),
                      
                      ACH = sum(t$stim==2 & t$disc==2 & t$aware==1),
                      UCH = sum(t$stim==2 & t$disc==2 & t$aware==0),
                      AIH = sum(t$stim==2 & t$disc==1 & t$aware==1),
                      UIH = sum(t$stim==2 & t$disc==1 & t$aware==0))
  
  # Generate ERPs
  sinSpace = seq(0,3*pi,0.1)
  plotSpace = seq(0,0.5,0.5/length(sinSpace))[1:95]
  erp_aware = sin(sinSpace)
  erp_unaware = erp_aware
  erp_unaware[length(seq(0,2*pi,0.1)):length(erp_unaware)] = 0
  
  ac1 = as.matrix(abs(t$ir[t$stim==1 & t$disc==1 & t$aware==1])) %*% erp_aware
  ac2 = as.matrix(abs(t$ir[t$stim==2 & t$disc==2 & t$aware==1])) %*% erp_aware
  ac = rbind(ac1,ac2)
  uc1 = as.matrix(abs(t$ir[t$stim==1 & t$disc==1 & t$aware==0])) %*% erp_unaware
  uc2 = as.matrix(abs(t$ir[t$stim==2 & t$disc==2 & t$aware==0])) %*% erp_unaware
  uc = rbind(uc1,uc2)
  return(list('ac' = ac, 'ac1' = ac1, 'ac2' = ac2, 'uc' = uc, 'uc1' = uc1, 'uc2' = uc2, 'trials' = trials,
              't' = t, 'plt' = plotSpace))
}

ERPplot = function(ptitle, s){
  # Plot ERPs
  ac = apply(s$ac,2,mean)
  uc = apply(s$uc,2,mean)
  ty1 = round(min(c(ac,uc)),1) - 0.1
  ty2 = round(max(c(ac,uc)),1) + 0.1
  plot(s$plt, ac, type = 'l', ylim = c(ty1,ty2), col = 'red', lwd = 3,
       xlab = 'Time (s)', ylab = 'V', 
       main = ptitle)
  points(s$plt, uc, type = 'l', col = 'blue', lwd = 3)
  abline(h = 0, v = 0)
  legend(x = 'bottomright', legend = c('Aware', 'Unaware'),
         col = c('red', 'blue'), lty = c(1,1), lwd = 3, bty = "n", cex = 0.5)
}

ERPcorrplot = function(ptitle, s, pylim = -1){
  # Plot ERPs corrected

  # Apply scaling factor
  ucc = (apply(s$uc1,2,mean)*s$trials$scaleL + apply(s$uc2,2,mean)*s$trials$scaleH)/2
  ac = apply(s$ac,2,mean)
  uc = apply(s$uc,2,mean)
  acc = ac - ucc
  if (pylim == -1){
    ty1 = round(min(c(ac,uc,ucc,acc)),1) - 0.1
    ty2 = round(max(c(ac,uc,ucc,acc)),1) + 0.1
    }
  if (pylim != -1){
    ty1 = -pylim
    ty2 = pylim
  }
  plot(s$plt, acc, type = 'l', ylim = c(ty1,ty2), col = 'red', lwd = 4,
       xlab = 'Time (s)', ylab = 'V',
       main = ptitle)
  points(s$plt, ucc, type = 'l', col = 'blue', lwd = 4)
  points(s$plt, ac, type = 'l', col = 'black', lwd = 4, lty = 2)
  points(s$plt, uc, type = 'l', col = 'black', lwd = 2, lty = 1)
  abline(h = 0, v = 0)
  legend(x = 'bottomright', legend = c('A minus U-corrected', 'U-corrected', 'A', 'U'),
         col = c('red', 'blue', 'black','black'), lty = c(1,1,2,1), lwd = c(4,4,4,2), bty = "n", cex = 1)
}

ERPgrandplot = function(ptitle, s, plt){
  # Plot ERP with 95% CIs
  
  simn = matrix(NA, 3, length(plt))
  for (i in 1:length(plt)){
    simn[1,i] = t.test(s[,i])$conf.int[2]
    simn[2,i] = t.test(s[,i])$estimate
    simn[3,i] = t.test(s[,i])$conf.int[1]
  }  
  ty1 = round(min(simn),1) - 0.1
  ty2 = round(max(simn),1) + 0.1
  ty1 = -2
  ty2 = 2
  plot(plt, simn[1,], type = 'l', ylim = c(ty1,ty2), col = 'black', lwd = 1, lty = 2,
       xlab = 'Time (s)', ylab = 'V',
       main = ptitle)
  points(plt, simn[2,], type = 'l', col = 'black', lwd = 2)
  points(plt, simn[3,], type = 'l', col = 'black', lwd = 1, lty = 2)
  abline(h = 0, v = 0)
  legend(x = 'bottomright', legend = c('95% CI UL', 'Mean (A minus U-corrected)', '95% CI LL'),
         col = c('black', 'black', 'black'), lty = c(1,1,1), lwd = c(1,2,1), bty = "n", cex = 0.8)
}

SDTdetect = function(d){
  # Compute alternative "detect" SDT model and scaling factors based on tone detection
  # high or low freq is considered as the signal whereas catch is the noise
  # (In terms of Macmillan & Creelman, 2005, low or high freq = S2 and catch = S1)
  # catch = stimulus A in figure 2 (left stimulus)
  # low or high freq = stimulus B in figure 2 (right stimulus)
  # With this labeling, the plotting matches that in M&C2005. Thus, criterion c is in the correct direction.
  
  # hit rate = P('Yes'|stimulus) - proportion of yes when stimulus present
  d$dHrL = (d$ACL+d$AIL)/(d$ACL+d$UCL+d$AIL+d$UIL) 
    # AIL: for a low tone, subject reported awareness but responded high freq
  d$dHrH = (d$ACH+d$AIH)/(d$ACH+d$UCH+d$AIH+d$UIH)
    # AIH: for a high tone, subject reported awareness but responded low freq
  
  # false alarm rate = P('Yes'|noise) - proportion of yes when noise
  # total number of catch trials = 80
  d$dFr = d$a_catch/80
  
  # D prime z(H) - z(F)
  d$ddprimeL = qnorm(d$dHrL) - qnorm(d$dFr)
  d$ddprimeH = qnorm(d$dHrH) - qnorm(d$dFr)
  
  # Criterion - 0.5 * (z(H) + z(F))
  d$dcL = - 0.5 * (qnorm(d$dHrL) + qnorm(d$dFr))
  d$dcH = - 0.5 * (qnorm(d$dHrH) + qnorm(d$dFr))
  # Positive value means conservative for detection (tendency not to report awareness)
  # Negative value means liberal for detection (tendency to report awareness)
  
  # means of distributions
  # (center them on zero and consider the distribution on the right)
  d$dmuL = d$ddprimeL/2
  d$dmuH = d$ddprimeH/2
  
  # weight of areas
  d$dawL = etruncnorm(a = d$dcL, b = Inf,  mean = d$dmuL)
  d$duwL = etruncnorm(a = -Inf, b = d$dcL, mean = d$dmuL)
  d$dawH = etruncnorm(a = d$dcH, b = Inf,  mean = d$dmuH)
  d$duwH = etruncnorm(a = -Inf, b = d$dcH, mean = d$dmuH)
  
  d$dscaleL = d$dawL/d$duwL
  d$dscaleH = d$dawH/d$duwH
  
  return(d)
}

SDTdetectplot = function(fp, da, whichplot = "B"){
  # detect: plot SDT representation

  x = seq(-da$ddprimeL/2 - 4, da$ddprimeL/2 + 4, 0.01)
  catchL = dnorm(x, mean = -da$ddprimeL/2) # noise
  catchH = dnorm(x, mean = -da$ddprimeH/2) # noise
  low = dnorm(x, mean = da$ddprimeL/2) # signal
  high = dnorm(x, mean = da$ddprimeH/2) # signal

  if(whichplot=="B"){ 
    par(mfrow=c(2,1)) #, pty="s")
  } 
  
  if(whichplot %in% c("L","B")){ 
    # low freq signal
  plot(x, catchL, type='l', xlab='Internal perceptual response', col='black',
       ylab='Density', main=paste('Subj = ', fp, 
                                  ", d' = ", round(da$ddprimeL, 2),
                                  ', c = ', round(da$dcL, 2), 
                                  ', scale factor low = ', round(da$dscaleL, 2), 
                                  sep = ''), xaxt='n')
  axis(side=1, at=round(min(x)):round(max(x)), labels=round(min(x)):round(max(x)))
  lines(x, low, type = 'l', col = 'cyan')
  abline(v = da$dcL, lty = 2)
  legend(x = "topleft",legend = c("catch","low freq","detection c"), col = c("black","cyan","black"), 
         lty=c(1,1,2), bty = "n", cex = 0.8)
  }

  if(whichplot %in% c("H","B")){ 
  # high freq signal
  plot(x, catchH, type='l', xlab='Internal perceptual response', col='black',
       ylab='Density', main=paste('Subj = ', fp, 
                                  ", d' = ", round(da$ddprimeH, 2),
                                  ', c = ', round(da$dcH, 2), 
                                  ', scale factor high = ', round(da$dscaleH, 2), 
                                  sep = ''), xaxt='n')
  axis(side=1, at=round(min(x)):round(max(x)), labels=round(min(x)):round(max(x)))
  lines(x, high, type = 'l', col = 'blue')
  abline(v = da$dcH, lty = 2)
  legend(x = "topleft",legend = c("catch","low freq","detection c"), col = c("black","blue","black"), 
         lty=c(1,1,2), bty = "n", cex = 0.8)
  }
  par(mfrow=c(1,1)) #, pty="s")
}


plotscale = function(dprime, c){
  # detect: plot scaling for simulated SDT indices
  
  simc = seq(-1.5, +1.5, 0.001)

  mu = rep(dprime/2,length(c))
  aw = etruncnorm(a = simc,    b = Inf, mean = mu)
  uw = etruncnorm(a = -Inf, b =   simc, mean = mu)

  scale = aw/uw

  par(mfrow=c(1,3)) #, pty="s")
  ty1 = round(min(c(uw, aw)),1) - 1
  ty2 = round(max(c(uw, aw)),1) + 1
  plot(simc, uw, type = 'l', ylim = c(ty1,ty2), col = 'blue', lwd = 3,
       xlab = 'Detection c', ylab = 'weight',
       main = paste0("Weights (d' = ", dprime, ")"))
  abline(v = c, lty = 2)
  lines(simc, aw, col = 'red', lwd = 3)
  abline(h = 0, lty = 1)
  abline(v = c, lty = 2)
  legend(x = 'topleft', legend = c('A', 'U', "Subject's c"),
         col = c('red', 'blue','black'), lty = c(1,1,2), lwd = c(3,3,1), bty = "n", cex = 1.5)

  plot(simc, scale, type = 'l', col = 'orange', lwd = 2,
       xlab = 'Detection c', ylab = 'Scaling factor',
       main = paste0("Scaling factor (d' = ", dprime, ")"))
  abline(h = 0, lty = 1)
  abline(v = c, lty = 2)
  
  plot(simc, scale, type = 'l', col = 'orange', lwd = 2,
       xlab = 'Detection c', ylab = 'Scaling factor',
       main = paste0("Scaling factor (d' = ", dprime, ", zoomed in)"),
       ylim = c(-10,10))
  abline(h = 0, lty = 1)
  abline(v = c, lty = 2)
  par(mfrow=c(1,1)) #, pty="s")

  return(list("scale"=scale, "aw"=aw, "uw"=uw, "dprime"=dprime, "c"=c))
  
}

plotmeanamps = function(dm, lbl = "label"){
  dm$A = rowMeans(cbind(dm$AcL, dm$AcH))
  dm$U = rowMeans(cbind(dm$UcL, dm$UcH))
  dm$Uc = rowMeans(cbind((dm$UcL * dm$scaleL), (dm$UcH * dm$scaleH)))
  dm$AmU = rowMeans(cbind((dm$AcL - dm$UcL),(dm$AcH - dm$UcH)))
  #dm$AmU2 = dm$A - dm$U
  dm$AmUc = rowMeans(cbind((dm$AcL - (dm$UcL * dm$scaleL)),  (dm$AcH - (dm$UcH * dm$scaleH))))
  #dm$AmUc2 = dm$A - dm$Uc
  dm$Diff = dm$AmU - dm$AmUc
  
  # ci = function(data){return(t.test(data)$conf.int[1:2])}

  dmdescr = data.frame("Condition" = c("A","U","Uc","AmU","AmUc","Diff"),
                  "N" = rep(nrow(dm),6),
                  "Mean" = apply(dm[,c("A","U","Uc","AmU","AmUc","Diff")],2,mean),
                  "CI_LL" = apply(dm[,c("A","U","Uc","AmU","AmUc","Diff")],2,ci)[1,],
                  "CI_UL" = apply(dm[,c("A","U","Uc","AmU","AmUc","Diff")],2,ci)[2,],
                  "SD" = apply(dm[,c("A","U","Uc","AmU","AmUc","Diff")],2,sd))
  dmdescr$Condition = factor(dmdescr$Condition, levels=c("A","U", "Uc", "AmU", "AmUc", "Diff"))
  
  dmg = gather(dm[,-c(2:7)], iv, dv, A, U, Uc, AmU, AmUc, Diff)
  dmg$fp = factor(dmg$fp)
  dmg$iv = factor(dmg$iv, levels=c("A","U", "Uc", "AmU", "AmUc", "Diff"))

  # https://stats.idre.ucla.edu/r/codefragments/ggplot2_errorbar/
  bp = ggplot(dmg, aes(x=iv, y=dv)) + 
    geom_jitter(width = 0.25, height = 0, shape=1, size=3) +
    theme_bw() + # get rid of background
    labs(title = paste0(lbl, " (N=",nrow(dm),")")) +
    theme(plot.title = element_text(hjust = 0.5)) +
    labs(x = "Variable") +
    labs(y = "Mean amp (V)") +
    geom_hline(yintercept = 0)+
    geom_errorbar(data = dmdescr, mapping = aes(x = Condition, y = Mean, ymin = CI_LL, ymax = CI_UL), size=1, color="black", width=.4)+
    geom_point(data=dmdescr, aes(x = Condition, y = Mean), size=3, color="black")
  plot(bp) 
  filename = paste0(lbl,"_amps")
  #ggsave(paste0(filename,'.png'), dpi = 300, width = 7, height = 4)
  
  # bpa = ggplot(dmdescr, aes(x=iv, y=Mean)) + 
  #   geom_bar(stat="identity"
  #            ,color="black" # add black border to each bar
  #            ,position=position_dodge()
  #            ,fill="#CCCCCC") +  
  #   theme_bw() + # get rid of background
  #   geom_errorbar(position=position_dodge(.9), width=.25, 
  #                 aes(ymin=CI_LL, ymax=CI_UL)) +
  #   labs(title = paste0(lbl, " (N=",nrow(dm),")")) +
  #   theme(plot.title = element_text(hjust = 0.5)) +
  #   labs(x = "Variable") +
  #   labs(y = "Mean amp (V)") +
  #   geom_hline(yintercept = 0)
  # plot(bpa) 
  # filename = paste0(lbl,"_means")
  # #ggsave(paste0(filename,'.png'), width = 4, height = 3)
  # 
  # 
  # bpb = ggplot(dmg, aes(x=iv, y=dv)) + 
  #   geom_jitter(width = 0.25, height = 0, shape=1, size=3) +
  #   theme_bw() + # get rid of background
  #   labs(title = paste0(lbl, " (N=",nrow(dm),")")) +
  #   theme(plot.title = element_text(hjust = 0.5)) +
  #   labs(x = "Variable") +
  #   labs(y = "Mean amp (V)") +
  #   geom_hline(yintercept = 0)
  # plot(bpb) 
  # filename = paste0(lbl,"_dots")
  # #ggsave(paste0(filename,'.png'), width = 4, height = 3)
  
  #write.csv2(dm, paste0(lbl,"_amps.csv"), sep = ";", dec = ".", row.names = FALSE)
  
  # separately return data values and descriptives
  return(list(dm,dmdescr))
}

plot2meanamps = function(d, yl='Y', xl='X', main='Title', spread=0.2, pch=1, size=2){
  jitter = function(s, e, x){return(sample(seq(s,e, (e-s)/x), x, F))}
  lim = function(d, r){return(c(min(d)-r, max(d)+r))}
  x = seq(0.1,0.9,0.8/(ncol(d)-1))
  dist = x[1]*spread
  plot(x=jitter(x[1]-dist, x[1]+dist, nrow(d)), y=d[,1], xlim=c(0,1),font.lab=2,
       xaxt='n', ylab=yl, xlab=xl, main=main, cex=size, ylim=lim(d, 1))#ylim=c(-15,15)) )
  for (i in 2:ncol(d)) {points(x=jitter(x[i]-dist, x[i]+dist, nrow(d)), d[,i],
                               cex=size)}
  axis(1, at=x, labels=colnames(d))
  abline(h=0)
  ci = function(data){return(t.test(data)$conf.int[1:2])}
  arrows(x, t(apply(d,2,ci))[,1], x, t(apply(d,2,ci))[,2],
         angle=90, code=3, length=dist*ncol(d)*2, col='black', lwd=4)
  points(x, apply(d,2,mean), col='black', pch=16, cex=size)
}
